import click
from .gateway_server import start_gateway

@click.group()
def cli():
    pass

@cli.command()
@click.option('--host', default='127.0.0.1', help='Host to run on')
@click.option('--port', default=9999, help='Port to run on')
@click.option('--targets', default=None, help='Path to targets.json')
def serve(host, port, targets):
    start_gateway(host=host, port=port, targets_path=targets)



class PrefixMiddleware:
    def __init__(self, app):
        self.app = app

    def __call__(self, environ, start_response):
        script_name = environ.get("HTTP_X_SCRIPT_NAME", "")
        if script_name:
            environ["SCRIPT_NAME"] = script_name
            path_info = environ.get("PATH_INFO", "")
            if path_info.startswith(script_name):
                environ["PATH_INFO"] = path_info[len(script_name):]
        return self.app(environ, start_response)


def gateway_patch(app):
    app.wsgi_app = PrefixMiddleware(app.wsgi_app)
    return app



from flask import Flask, request, Response, render_template
import requests
import json

app = Flask(__name__)
TARGETS = {}

@app.route("/")
def home():
    return render_template('index.html', targets=TARGETS)

@app.route('/<app>/', methods=['GET', 'POST', 'PUT', 'DELETE'])
@app.route('/<app>/<path:path>', methods=['GET', 'POST', 'PUT', 'DELETE'])
def proxy(app, path=''):
    target_config = TARGETS.get(app)
    if not target_config:
        return 'Unknown proxy target', 404

    base = target_config['target']
    prefix = f"/{app}"
    url = f"{base}/{path}"

    headers = {
      key: value for key, value in request.headers if key.lower() != 'host'
    }
    headers['X-Script-Name'] = prefix

    resp = requests.request(
        method=request.method,
        url=url,
        headers=headers,
        data=request.get_data(),
        cookies=request.cookies,
        allow_redirects=False
    )


    excluded_headers = ['content-encoding', 'content-length', 'transfer-encoding', 'connection']
    headers = [(k, v) for k, v in resp.raw.headers.items() if k.lower() not in excluded_headers]

    return Response(resp.content, resp.status_code, headers)

def start_gateway(host='127.0.0.1', port=9999, targets_path=None):
    if targets_path:
        global TARGETS
        with open(targets_path) as f:
            TARGETS = json.load(f)

    app.run(host=host, port=port, debug=True)

if __name__ == "__main__":
    start_gateway()
